/*
 * Copyright (c) 2004 Virtual Worlds Productions
 * Email: virtual_worlds@gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * Please contact virtual_worlds@gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.GraphicsConfiguration;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import com.sun.j3d.utils.image.*;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Universe extends Applet
   {
   private SimpleUniverse u=null;
   private BoundingSphere BigBounds=new BoundingSphere(new Point3d(),100000);


   public Universe()
      {
      }


   void createEnvironment(BranchGroup bg)
      {
      Background       BG=new Background();

      BG.setColor(new Color3f(1.0f,1.0f,1.0f));
      BG.setApplicationBounds(new BoundingSphere(new Point3d(),5.0));
      bg.addChild(BG);
      }



   void createSoundNodes(BranchGroup bg)
      {
      MediaContainer  BGMC;
      BackgroundSound BGSnd;

      BGMC=new MediaContainer("file:./voegel.wav");
      BGSnd=new BackgroundSound(BGMC,1.0f,-1,false,true,true,BigBounds,0);
      bg.addChild(BGSnd);
      }



   void createObjects(BranchGroup bg)
      {
      Shape3D       S3D=new Shape3D();
      Point3f[]     CoordArr=new Point3f[4];
      TexCoord2f[]  TexCoordArr=new TexCoord2f[4];
      QuadArray     QuadArr=new QuadArray(4,QuadArray.COORDINATES|QuadArray.TEXTURE_COORDINATE_2);
      Appearance    S3DApp=new Appearance();

      CoordArr[0]=new Point3f(-17.5f,-1.5f,-17.5f);
      CoordArr[1]=new Point3f(-17.5f,-1.5f, 17.5f);
      CoordArr[2]=new Point3f( 17.5f,-1.5f, 17.5f);
      CoordArr[3]=new Point3f( 17.5f,-1.5f,-17.5f);

      TexCoordArr[0]=new TexCoord2f( 0f,40f);
      TexCoordArr[1]=new TexCoord2f( 0f, 0f);
      TexCoordArr[2]=new TexCoord2f(40f, 0f);
      TexCoordArr[3]=new TexCoord2f(40f,40f);

      QuadArr.setCoordinates(0,CoordArr);
      QuadArr.setTextureCoordinates(0,0,TexCoordArr);
      S3D.addGeometry(QuadArr);
      S3DApp.setMaterial(new Material(new Color3f(1.0f,0f,0f),new Color3f(0f,1f,0f),new Color3f(1.0f,0f,0f),new Color3f(1.0f,0f,0f),10f));
      S3DApp.setTexture((new TextureLoader("teppich.png",null)).getTexture());
      S3DApp.getTexture().setMagFilter(Texture.NICEST);
      S3DApp.getTexture().setMinFilter(Texture.NICEST);
      S3DApp.setTextureAttributes(new TextureAttributes(TextureAttributes.MODULATE,new Transform3D(),new Color4f(),TextureAttributes.NICEST));
      S3D.setAppearance(S3DApp);
      bg.addChild(S3D);
      }


   public BranchGroup createSceneGraph()
      {
      BranchGroup          RootBG=new BranchGroup();
      TransformGroup       ViewTG=new TransformGroup();
      Appearance           BoxAppearance=new Appearance();
      DirectionalLight     DLgt=new DirectionalLight(new Color3f(0.8f,0.8f,0.9f),new Vector3f(-0.5f,-1f,-0.5f));
      AmbientLight         ALgt=new AmbientLight(new Color3f(0.8f,0.8f,0.8f));

      ALgt.setInfluencingBounds(BigBounds);
      DLgt.setInfluencingBounds(BigBounds);
      RootBG.addChild(ALgt);
      RootBG.addChild(DLgt);

      ViewTG=u.getViewingPlatform().getViewPlatformTransform();
      MouseRotate MouseCtrl=new MouseRotate(ViewTG);
	   MouseCtrl.setSchedulingBounds(BigBounds);
      RootBG.addChild(MouseCtrl);
      MouseZoom MouseCtrl2=new MouseZoom(ViewTG);
	   MouseCtrl2.setSchedulingBounds(BigBounds);
      RootBG.addChild(MouseCtrl2);

      createEnvironment(RootBG);
      createObjects(RootBG);
      createSoundNodes(RootBG);
      RootBG.compile();

      return RootBG;
      }



   void setViewPosition()
      {
      TransformGroup ViewTG;
      Transform3D    ViewT3D=new Transform3D();

      ViewTG=u.getViewingPlatform().getViewPlatformTransform();
      ViewTG.getTransform(ViewT3D);
      ViewT3D.setTranslation(new Vector3f(0f,0f,4f));
      ViewTG.setTransform(ViewT3D);
      }



   public void init()
      {
	   setLayout(new BorderLayout());
      GraphicsConfiguration config=SimpleUniverse.getPreferredConfiguration();
      Canvas3D c = new Canvas3D(config);
	   add("Center", c);
      u=new SimpleUniverse(c);

      AudioDevice auDev=u.getViewer().createAudioDevice();
      auDev.setAudioPlaybackType(AudioDevice.STEREO_SPEAKERS);

      setViewPosition();
      u.addBranchGraph(createSceneGraph());
      }



   public void destroy()
      {
//	   u.cleanup();
      }

   public static void main(String[] args)
      {
	   new MainFrame(new Universe(), 600, 400);
      }





}
